/*
 * This file is part of the KFTPGrabber project
 *
 * Copyright (C) 2003-2004 by the KFTPGrabber developers
 * Copyright (C) 2003-2004 Jernej Kos <kostko@jweb-network.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * is provided AS IS, WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, and
 * NON-INFRINGEMENT.  See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA.
 *
 * In addition, as a special exception, the copyright holders give
 * permission to link the code of portions of this program with the
 * OpenSSL library under certain conditions as described in each
 * individual source file, and distribute linked combinations
 * including the two.
 * You must obey the GNU General Public License in all respects
 * for all of the code used other than OpenSSL.  If you modify
 * file(s) with this exception, you may extend this exception to your
 * version of the file(s), but you are not obligated to do so.  If you
 * do not wish to do so, delete this exception statement from your
 * version.  If you delete this exception statement from all source
 * files in the program, then also delete it here.
 */

#include "logview.h"
#include "misc/config.h"

#include <klocale.h>
#include <kstandardaction.h>
#include <kfiledialog.h>
#include <kmessagebox.h>
#include <KGlobal>
#include <KComponentData>
#include <KAboutData>

#include <QFile>
#include <QTextStream>
#include <QScrollBar>
#include <QFont>
#include <QKeyEvent>
#include <QPainter>

namespace KFTPWidgets {

LogView::LogView(QWidget *parent)
  : QAbstractScrollArea(parent),
    m_numberOfLines(0),
    m_maxLineWidth(0),
    m_limit(200)
{
  m_curFont = KFTPCore::Config::logFont();
  m_fontsize = QFontMetrics(m_curFont).lineSpacing();

  setAttribute(Qt::WA_StaticContents);
  setAttribute(Qt::WA_NoBackground);
  setUpdatesEnabled(true);
  setFrameStyle(QFrame::NoFrame);
  
  verticalScrollBar()->setMaximum(m_content.size() * m_fontsize);
  verticalScrollBar()->setPageStep(int(height() * 0.8));
  verticalScrollBar()->setSingleStep(m_fontsize);   
  horizontalScrollBar()->setPageStep(int(width() * 0.8));
  
  // Init actions
  m_saveToFileAction = KStandardAction::saveAs(this, SLOT(slotSaveToFile()), this);
  m_clearLogAction = KStandardAction::clear(this, SLOT(clear()), this);
}

LogView::~LogView()
{
}

void LogView::clear() 
{
  m_content.clear();
  m_numberOfLines = 0;
  m_maxLineWidth = 0;
  updateScrollbar();
  viewport()->update();
}

void LogView::append(const QString &str, LineType type)
{
  Line line;
  line.type = type;
  line.text = str.trimmed();
  
  switch (type) {
    case FtpResponse: {
      // Differentiate between code and message
      QString prefix = line.text.section(" ", 0, 0);
      QString text = line.text.mid(line.text.indexOf(' '));
      
      m_curFont.setBold(true);
      line.width = QFontMetrics(m_curFont).width(prefix);
      m_curFont.setBold(false);
      line.width += QFontMetrics(m_curFont).width(text);
      
      // Prefix
      QTextCharFormat cf;
      cf.setFontWeight(QFont::Bold);
      cf.setForeground(QBrush(KFTPCore::Config::logResponsesColor()));
      
      QTextLayout::FormatRange fr;
      fr.format = cf;
      fr.start = 0;
      fr.length = prefix.length();
      line.formatRanges << fr;
      
      // Message text
      cf.setFontWeight(QFont::Normal);
      fr.format = cf;
      fr.start = prefix.length() + 1;
      fr.length = text.length();
      line.formatRanges << fr;
      break;
    }
    case FtpCommand: {
      // Hide password first if this is a PASS command
      if (line.text.left(4) == "PASS") {
        line.text = "PASS (hidden)";
      }
      
      m_curFont.setBold(true);
      line.width = QFontMetrics(m_curFont).width(line.text);
      m_curFont.setBold(false);
      
      // Format
      QTextCharFormat cf;
      cf.setFontWeight(QFont::Bold);
      cf.setForeground(QBrush(KFTPCore::Config::logCommandsColor()));
      
      QTextLayout::FormatRange fr;
      fr.format = cf;
      fr.start = 0;
      fr.length = line.text.length();
      line.formatRanges << fr;
      break;
    }
    case FtpMultiline: {
      line.width = QFontMetrics(m_curFont).width(line.text);
      
      // Format
      QTextCharFormat cf;
      cf.setForeground(QBrush(KFTPCore::Config::logMultilineColor()));
      
      QTextLayout::FormatRange fr;
      fr.format = cf;
      fr.start = 0;
      fr.length = line.text.length();
      line.formatRanges << fr;
      break;
    }
    case FtpStatus: {
      line.text.prepend("*** ");
      
      m_curFont.setBold(true);
      line.width = QFontMetrics(m_curFont).width(line.text);
      m_curFont.setBold(false);
      
      // Format
      QTextCharFormat cf;
      cf.setFontWeight(QFont::Bold);
      cf.setForeground(QBrush(KFTPCore::Config::logStatusColor()));
      
      QTextLayout::FormatRange fr;
      fr.format = cf;
      fr.start = 0;
      fr.length = line.text.length();
      line.formatRanges << fr;
      break;
    }
    case FtpError: {
      line.text.prepend("*** ");
      
      m_curFont.setBold(true);
      line.width = QFontMetrics(m_curFont).width(line.text);
      m_curFont.setBold(false);
      
      // Format
      QTextCharFormat cf;
      cf.setFontWeight(QFont::Bold);
      cf.setForeground(QBrush(KFTPCore::Config::logErrorColor()));
      
      QTextLayout::FormatRange fr;
      fr.format = cf;
      fr.start = 0;
      fr.length = line.text.length();
      line.formatRanges << fr;
      break;
    }
    default: line.width = QFontMetrics(m_curFont).width(line.text); break;
  }
  
  manageLimit();
  m_content << line;
  m_numberOfLines = m_content.size();
  
  if (line.width > m_maxLineWidth)
    m_maxLineWidth = line.width;

  updateScrollbar();
  
  // Set the scrollbar position at the bottom of the widget
  if (!verticalScrollBar()->isSliderDown() && !horizontalScrollBar()->isSliderDown()) {
    verticalScrollBar()->setValue(verticalScrollBar()->maximum());
    viewport()->repaint();
  }
}

void LogView::manageLimit()
{
  if (m_content.size() >= m_limit) {
    Line first = m_content.takeFirst();
    
    if (first.width == m_maxLineWidth) {
      // Find a new maximum line width - this is O(n)
      m_maxLineWidth = 0;
      
      foreach (Line line, m_content) {
        if (line.width > m_maxLineWidth)
          m_maxLineWidth = line.width;
      }
    }
  }
}

void LogView::updateScrollbar()
{
  int max = (m_numberOfLines * m_fontsize) - viewport()->height();
  verticalScrollBar()->setMaximum(max < 0 ? 0 : max);
  
  max = m_maxLineWidth - viewport()->width();
  horizontalScrollBar()->setMaximum(max < 0 ? 0 : max);
}

void LogView::keyPressEvent(QKeyEvent *e)
{
  switch (e->key()) {
    case Qt::Key_PageUp: {
      verticalScrollBar()->triggerAction(QAbstractSlider::SliderPageStepSub);
      break;
    }
    case Qt::Key_PageDown: {
      verticalScrollBar()->triggerAction(QAbstractSlider::SliderPageStepAdd);
      break;
    }
    default: e->ignore(); break;
  }
}

void LogView::resizeEvent(QResizeEvent *event)
{
  Q_UNUSED(event)
  
  updateScrollbar();
  verticalScrollBar()->setPageStep(int(height() * 0.8));
  horizontalScrollBar()->setPageStep(int(width() * 0.8));
}

void LogView::paintEvent(QPaintEvent *event)
{
  QPainter p(viewport());
  paint(&p, event);
}

void LogView::paint(QPainter *p,  QPaintEvent *e)
{
  const int xOffset = horizontalScrollBar()->value();
  const int yOffset = verticalScrollBar()->value();

  QRect r = e->rect();
  p->translate(-xOffset, -yOffset);
  r.translate(xOffset, yOffset);
  p->setClipRect(r);

  p->setPen(Qt::black);
  
  for (int i = (r.top() / m_fontsize); i < (r.bottom() / m_fontsize) + 1; i++) {
    if (i < m_content.size()) {
      Line line = m_content[i];
      
      QTextLayout textLayout(m_content[i].text, m_curFont);
      textLayout.setAdditionalFormats(line.formatRanges);
      textLayout.beginLayout();
      textLayout.createLine();
      textLayout.endLayout();

      textLayout.draw(p, QPointF(0, i * m_fontsize));
    }
  }
}

void LogView::contextMenuEvent(QContextMenuEvent *event)
{
  QMenu menu;
  
  menu.addAction(m_saveToFileAction);
  menu.addSeparator();
  menu.addAction(m_clearLogAction);
  menu.exec(event->globalPos());
}

void LogView::slotSaveToFile()
{
  QString savePath = KFileDialog::getSaveFileName();
  
  if (!savePath.isEmpty()) {
    QFile file(savePath);
    if (file.open(QIODevice::WriteOnly)) {
      QTextStream stream(&file);
      
      for (int i = 0; i < m_content.size(); ++i) {
        stream << m_content.at(i).text << endl;
      }
      
      file.close();
    } else {
      KMessageBox::error(0L, i18n("Unable to open file for writing."));
    }
  }
}

}

#include "logview.moc"
